package edu.sc.seis.cormorant.event;

import org.omg.CORBA.CompletionStatus;

import edu.iris.Fissures.IfEvent.EventAccess;
import edu.iris.Fissures.IfEvent.EventAccessHelper;
import edu.iris.Fissures.IfEvent.EventChannelFinder;
import edu.iris.Fissures.IfEvent.EventFactory;
import edu.iris.Fissures.IfEvent.EventFinder;
import edu.iris.Fissures.IfEvent.EventFinderPOA;
import edu.iris.Fissures.IfEvent.EventSeqIterHolder;
import edu.sc.seis.fissuresUtil.cache.CacheEvent;
import edu.sc.seis.fissuresUtil.display.MicroSecondTimeRange;
import edu.sc.seis.fissuresUtil.flow.querier.EventFinderQuery;

public class EventFinderImpl extends EventFinderPOA {

    public EventFinderImpl(EventDataAccess eventdb,
                           org.omg.PortableServer.POA poa) {
        this.poa = poa;
        this.eventdb = eventdb;
    }

    public EventAccess[] query_events(edu.iris.Fissures.Area the_area,
                                      edu.iris.Fissures.Quantity min_depth,
                                      edu.iris.Fissures.Quantity max_depth,
                                      edu.iris.Fissures.TimeRange time_range,
                                      String[] search_types,
                                      float min_magnitude,
                                      float max_magnitude,
                                      String[] catalogs,
                                      String[] contributors,
                                      int seq_max,
                                      EventSeqIterHolder iter) {
        EventAccess[] eventAccess;
        logger.debug("In the method query_events of EventFinder: "+the_area.getClass());
        logger.debug("Depth: "+min_depth.value+" "+max_depth.value);
        logger.debug("Time: "+time_range.start_time.date_time+" "+time_range.end_time.date_time);
        logger.debug("Map: "+min_magnitude+" "+max_magnitude);
        logger.debug("Catalog: "+catalogs.length+" "+(catalogs.length>0?catalogs[0]:""));
        logger.debug("Contributor: "+contributors.length);
        int counter;
        try {
            EventFinderQuery eventFinderQuery = new EventFinderQuery();
            eventFinderQuery.setArea(the_area);
            eventFinderQuery.setMinDepth(min_depth.value);
            eventFinderQuery.setMaxDepth(max_depth.value);
            eventFinderQuery.setMinMag(min_magnitude);
            eventFinderQuery.setMaxMag(max_magnitude);
            eventFinderQuery.setTime(new MicroSecondTimeRange(time_range));
            if(catalogs.length == 1 && catalogs[0].equalsIgnoreCase("PREF")) {
                eventFinderQuery.setCatalogs(new String[0]);
            } else {
                eventFinderQuery.setCatalogs(catalogs);
            }
            int[] events = eventdb.query(eventFinderQuery);
            eventAccess = new EventAccess[events.length];
            for(counter = 0; counter < eventAccess.length; counter++) {
                org.omg.CORBA.Object obj = poa.create_reference_with_id(("" + events[counter]).getBytes(),
                                                                        EventAccessHelper.id());
                eventAccess[counter] = EventAccessHelper.narrow(obj);
            }
            logger.debug("found "+eventAccess.length+" events");
            return eventAccess;
        } catch(Throwable e) {
            logger.error("Generic Exception ", e);
            throw new org.omg.CORBA.UNKNOWN(e.toString(),
                                            1,
                                            CompletionStatus.COMPLETED_NO); 
        }
    }

    public EventAccess[] get_by_name(String name) {
        CacheEvent[] eventIds;
        int counter;
        EventAccess[] eventAccess;
        try {
            eventIds = eventdb.getByName(name);
            eventAccess = new EventAccess[eventIds.length];
            for(counter = 0; counter < eventIds.length; counter++) {
                org.omg.CORBA.Object obj = poa.create_reference_with_id(("" + eventIds[counter].getDbid()).getBytes(),
                                                                        EventAccessHelper.id());
                eventAccess[counter] = EventAccessHelper.narrow(obj);
                String ior = _orb().object_to_string(obj);
            }
            return eventAccess;
        } catch(Exception e) {
            logger.error("Generic Exception ", e);
            throw new org.omg.CORBA.UNKNOWN(e.toString());
        }
    }

    public EventFactory a_factory() {
        return eventFactory;
    }

    public EventFinder a_finder() {
        return eventFinder;
    }

    public EventChannelFinder a_channel_finder() {
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    public String[] known_catalogs() {
        return eventdb.getCatalogs();
    }

    public String[] known_contributors() {
        return eventdb.getContributors();
    }

    public String[] catalogs_from(String contributor) {
        return eventdb.getCatalogsFor(contributor);
    }

    public void setEventFinder(EventFinder eventFinder) {
        this.eventFinder = eventFinder;
    }

    public void setEventFactory(EventFactory eventFactory) {
        this.eventFactory = eventFactory;
    }

    private EventFinder eventFinder = null;

    private EventFactory eventFactory = null;

    protected EventDataAccess eventdb;

    org.omg.PortableServer.POA poa;

    private static final org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(EventFinderImpl.class);
} // EventFinderImpl
