package edu.sc.seis.cormorant.network;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import edu.iris.Fissures.IfNetwork.Channel;
import edu.iris.Fissures.IfNetwork.ChannelGroupIterHolder;
import edu.iris.Fissures.IfNetwork.ChannelId;
import edu.iris.Fissures.IfNetwork.ChannelIdIterHolder;
import edu.iris.Fissures.IfNetwork.ChannelNotFound;
import edu.iris.Fissures.IfNetwork.NetworkExplorerPOA;
import edu.iris.Fissures.IfNetwork.OrientationRange;
import edu.iris.Fissures.IfNetwork.SamplingRange;
import edu.iris.Fissures.network.ChannelImpl;
import edu.iris.Fissures.network.StationImpl;
import edu.sc.seis.fissuresUtil.bag.AreaUtil;
import edu.sc.seis.fissuresUtil.bag.OrientationUtil;
import edu.sc.seis.fissuresUtil.bag.SamplingUtil;
import edu.sc.seis.fissuresUtil.database.NotFound;
import edu.sc.seis.fissuresUtil.display.DisplayUtils;
import edu.sc.seis.fissuresUtil.hibernate.NetworkDB;

/**
 * NetworkExplorerImpl.java Created: Thu Mar 15 11:35:13 2001
 * 
 * @author Philip Crotwell
 */
public class NetworkExplorerImpl extends NetworkExplorerPOA {

    public NetworkExplorerImpl(NetworkDB jdbcChannel) {
        this.netdb = jdbcChannel;
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkExplorer/locate_channels:1.0
    //
    /***/
    public ChannelId[] locate_channels(edu.iris.Fissures.Area the_area,
                                       SamplingRange sampling,
                                       OrientationRange orientation,
                                       int seq_max,
                                       ChannelIdIterHolder iter) {
        try {
            List<ChannelImpl> chans = AreaUtil.inArea(the_area, netdb.getAllChannels());
            chans = SamplingUtil.inSampling(sampling, chans);
            chans = OrientationUtil.inOrientation(orientation,
                                                  chans);
            return ConcreteNetworkAccessImpl.getIds((ChannelImpl[])chans.toArray(new ChannelImpl[0]));
        } catch(Throwable e) {
            logger.error("locate_channels ", e);
            throw new org.omg.CORBA.UNKNOWN(e.getMessage());
        }
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkExplorer/locate_by_sampling:1.0
    //
    /***/
    public ChannelId[][] locate_by_sampling(edu.iris.Fissures.Area the_area,
                                            SamplingRange sampling,
                                            int seq_max,
                                            ChannelGroupIterHolder iter) {
        try {
            List<ChannelImpl> chans = AreaUtil.inArea(the_area, netdb.getAllChannels());
            return group(SamplingUtil.inSampling(sampling, chans));
        } catch(Throwable e) {
            logger.error("locate_by_sampling ", e);
            throw new org.omg.CORBA.UNKNOWN(e.getMessage());
        }
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkExplorer/locate_all:1.0
    //
    /***/
    public ChannelId[][] locate_all(edu.iris.Fissures.Area the_area,
                                    int seq_max,
                                    ChannelGroupIterHolder iter) {
        try {
            return group(AreaUtil.inArea(the_area, netdb.getAllChannels()));
        } catch(Throwable e) {
            logger.error("locate_all ", e);
            throw new org.omg.CORBA.UNKNOWN(e.getMessage());
        }
    }

    private static ChannelId[][] group(List<ChannelImpl> chans) {
        ChannelId[][] chanIds = new ChannelId[chans.size()][1];
        int i=0;
        for(ChannelImpl chan : chans) {
            chanIds[i][0] = chan.get_id();
            i++;
        }
        return chanIds;
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkExplorer/retrieve_channel:1.0
    //
    /***/
    public Channel retrieve_channel(ChannelId a_channel) throws ChannelNotFound {
        try {
            return netdb.getChannel(a_channel);
        } catch(NotFound e) {
            throw new ChannelNotFound(e.getMessage(), a_channel);
        }
    }

    //
    // IDL:iris.edu/Fissures/IfNetwork/NetworkExplorer/retrieve_grouping:1.0
    //
    /***/
    public ChannelId[] retrieve_grouping(ChannelId a_channel)
            throws ChannelNotFound {
        //get them out of the DB...
        List<StationImpl> staList = netdb.getStationByCodes(a_channel.network_id.network_code, a_channel.station_code);
        List<ChannelImpl> allChans = new ArrayList<ChannelImpl>();
        for(StationImpl sta : staList) {
            List<ChannelImpl> staChan = netdb.getChannelsForStation(sta);
            allChans.addAll(staChan);
        }
        ChannelId[] out = new ChannelId[3];
        int j = 0;
        for(ChannelImpl channelImpl : allChans) {
            if(DisplayUtils.areFriends(a_channel, channelImpl.get_id())) {
                out[j] = channelImpl.get_id();
                j++;
            }
        }
        if(j == 0) { throw new ChannelNotFound(); }
        return out;
    }

    protected NetworkDB netdb;

    private static final Logger logger = LoggerFactory.getLogger(NetworkExplorerImpl.class);
} // NetworkExplorerImpl
