package edu.sc.seis.cormorant.network;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.omg.CORBA.OBJECT_NOT_EXIST;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.Servant;
import org.omg.PortableServer.ServantLocatorPOA;
import org.omg.PortableServer.ServantLocatorPackage.CookieHolder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import edu.iris.Fissures.IfNetwork.ConcreteNetworkAccessHelper;
import edu.iris.Fissures.IfNetwork.ConcreteNetworkAccessPOA;
import edu.iris.Fissures.IfNetwork.NetworkAccess;
import edu.iris.Fissures.IfNetwork.NetworkId;
import edu.iris.Fissures.network.NetworkAttrImpl;
import edu.iris.Fissures.network.NetworkIdUtil;
import edu.sc.seis.fissuresUtil.database.NotFound;
import edu.sc.seis.fissuresUtil.hibernate.NetworkDB;

/**
 * NetworkServantLocator.java
 * 
 * 
 * Created: Thu Mar 15 12:27:25 2001
 * 
 * @author Philip Crotwell
 */
public class NetworkServantLocator extends ServantLocatorPOA {

    public NetworkServantLocator(POA myPOA)
            throws NotFound {
        this.myPOA = myPOA;
        refresh();
    }

    public Servant preinvoke(byte[] oid,
                             POA adapter,
                             String operation,
                             CookieHolder the_cookie) {
        String idString = new String(oid);
        logger.debug("NetworkServantLocator: " + idString + " " + operation);
        if(idString.endsWith("NetworkAccess")) {
            return getNetworkAccessPOA(idString);
        } else {
            throw new OBJECT_NOT_EXIST();
        }
    }

    public void postinvoke(byte[] oid,
                           POA adapter,
                           String operation,
                           Object the_cookie,
                           Servant the_servant) {}

    NetworkAccess getNetworkAccess(NetworkId networkId) {
        String networkObjId = NetworkIdUtil.toString(networkId);
        String key = networkIdToObjKey(networkId);
        return getNetworkAccess(key);
    }

    NetworkAccess getNetworkAccess(String key) {
        org.omg.CORBA.Object obj = null;
        // try {
        obj = myPOA.create_reference_with_id(key.getBytes(),
                                             ConcreteNetworkAccessHelper.id());
        // } catch (WrongPolicy e) {
        // // shouldn't happen???
        // logger.error("Wrong policy on POA:", e);
        // throw new org.omg.CORBA.UNKNOWN(e.getMessage());
        // }
        return ConcreteNetworkAccessHelper.narrow(obj);
    }

    public synchronized void refresh() throws NotFound {
        networks.clear();
        List<NetworkAttrImpl> allNets = NetworkDB.getSingleton().getAllNetworks();
        for(NetworkAttrImpl networkAttrImpl : allNets) {
            networks.put(networkIdToObjKey(networkAttrImpl.get_id()), networkAttrImpl);
        }
    }

    public String networkIdToObjKey(NetworkId id) {
        return id.network_code + "_" + id.begin_time.date_time
                + "_NetworkAccess";
    }

    public String objKeyToNetworkCode(String objKey) {
        return objKey.substring(0, 2);
    }

    public NetworkId objKeyToNetworkId(String objKey) {
        if (networks.containsKey(objKey)) {
            return networks.get(objKey).get_id();
        }
        logger.warn("Object key does not exist: "+objKey);
        throw new OBJECT_NOT_EXIST();
    }

    protected ConcreteNetworkAccessPOA getNetworkAccessPOA(String networkObjId) {
        if(!networkAccessPoaMap.containsKey(networkObjId)) {
            networkAccessPoaMap.put(networkObjId,
                                    new ConcreteNetworkAccessImpl(objKeyToNetworkId(networkObjId)));
        }
        return (ConcreteNetworkAccessImpl)networkAccessPoaMap.get(networkObjId);
    }

    /**
     * @returns an iterator over all of the network object ids of
     *          NetworkAccessPOAs know.
     */
    Iterator getNetworkIterator() {
        return networks.keySet().iterator();
    }

    protected HashMap<String, NetworkAttrImpl> networks = new HashMap<String, NetworkAttrImpl>();

    protected HashMap<String, ConcreteNetworkAccessPOA> networkAccessPoaMap = new HashMap<String, ConcreteNetworkAccessPOA>();

    protected POA myPOA;

    private static Logger logger = LoggerFactory.getLogger(NetworkServantLocator.class.getName());
} // NetworkServantLocator
